@@ -28,7 +28,6 @@ def as_high_level_wcs(wcs):
 @pytest.mark.parametrize("center_jacobian", (False, True))
 @pytest.mark.parametrize("roundtrip_coords", (False, True))
 def test_reproject_adaptive_2d(wcsapi, center_jacobian, roundtrip_coords):
-
     # Set up initial array with pattern
     data_in = np.zeros((256, 256))
     data_in[::20, :] = 1
@@ -64,12 +63,6 @@ def test_reproject_adaptive_2d(wcsapi, center_jacobian, roundtrip_coords):
     # Check that surface brightness is conserved in the unrotated case
     assert_allclose(np.nansum(data_in), np.nansum(array_out) * (256 / 60) ** 2, rtol=0.1)
 
-    # ASTROPY_LT_40: astropy v4.0 introduced new default header keywords,
-    # once we support only astropy 4.0 and later we can update the reference
-    # data files and remove this section.
-    for key in ("DATEREF", "MJDREFF", "MJDREFI", "MJDREF", "MJD-OBS"):
-        header_out.pop(key, None)
-
     return array_footprint_to_hdulist(array_out, footprint_out, header_out)
 
 
@@ -78,7 +71,6 @@ def test_reproject_adaptive_2d(wcsapi, center_jacobian, roundtrip_coords):
 @pytest.mark.parametrize("center_jacobian", (False, True))
 @pytest.mark.parametrize("roundtrip_coords", (False, True))
 def test_reproject_adaptive_2d_rotated(center_jacobian, roundtrip_coords):
-
     # Set up initial array with pattern
     data_in = np.zeros((256, 256))
     data_in[::20, :] = 1
@@ -108,12 +100,6 @@ def test_reproject_adaptive_2d_rotated(center_jacobian, roundtrip_coords):
         boundary_mode="ignore",
     )
 
-    # ASTROPY_LT_40: astropy v4.0 introduced new default header keywords,
-    # once we support only astropy 4.0 and later we can update the reference
-    # data files and remove this section.
-    for key in ("DATEREF", "MJDREFF", "MJDREFI", "MJDREF", "MJD-OBS"):
-        header_out.pop(key, None)
-
     return array_footprint_to_hdulist(array_out, footprint_out, header_out)
 
 
@@ -155,7 +141,7 @@ def test_reproject_adaptive_high_aliasing_potential_rotation(
     # along the output x axis. With the input image containing vertical lines
     # with values of zero or one, we should have uniform values of 0.5
     # throughout our output array.
-    np.testing.assert_allclose(array_out, 0.5, rtol=0.001)
+    np.testing.assert_allclose(array_out, 0.5, rtol=0.002)
 
     # Within the transforms, the order of operations is:
     # input pixel coordinates -> input rotation -> input scaling
@@ -175,7 +161,7 @@ def test_reproject_adaptive_high_aliasing_potential_rotation(
         center_jacobian=center_jacobian,
         kernel=kernel,
     )
-    np.testing.assert_allclose(array_out, 0.5, rtol=0.001)
+    np.testing.assert_allclose(array_out, 0.5, rtol=0.002)
 
     # But if we add a 90-degree rotation to the input coordinates, then when
     # our stretched output pixels are projected onto the input data, they will
@@ -669,7 +655,6 @@ def prepare_test_data(file_format):
 @pytest.mark.array_compare(single_reference=True)
 @pytest.mark.parametrize("file_format", ["fits", "asdf"])
 def test_reproject_adaptive_roundtrip(file_format):
-
     # Test the reprojection with solar data, which ensures that the masking of
     # pixels based on round-tripping works correctly. Using asdf is not just
     # about testing a different format but making sure that GWCS works.
@@ -682,22 +667,6 @@ def test_reproject_adaptive_roundtrip(file_format):
 
     header_out = target_wcs.to_header()
 
-    # ASTROPY_LT_40: astropy v4.0 introduced new default header keywords,
-    # once we support only astropy 4.0 and later we can update the reference
-    # data files and remove this section.
-    for key in (
-        "CRLN_OBS",
-        "CRLT_OBS",
-        "DSUN_OBS",
-        "HGLN_OBS",
-        "HGLT_OBS",
-        "MJDREFF",
-        "MJDREFI",
-        "MJDREF",
-        "MJD-OBS",
-        "RSUN_REF",
-    ):
-        header_out.pop(key, None)
     header_out["DATE-OBS"] = header_out["DATE-OBS"].replace("T", " ")
 
     return array_footprint_to_hdulist(output, footprint, header_out)
@@ -706,7 +675,6 @@ def test_reproject_adaptive_roundtrip(file_format):
 @pytest.mark.filterwarnings("ignore::FutureWarning")
 @pytest.mark.array_compare()
 def test_reproject_adaptive_uncentered_jacobian():
-
     # Explicitly test the uncentered-Jacobian path for a non-affine transform.
     # For this case, output pixels change by 6% at most, and usually much less.
     # (Though more nan pixels are present, as the uncentered calculation draws
@@ -720,22 +688,6 @@ def test_reproject_adaptive_uncentered_jacobian():
 
     header_out = target_wcs.to_header()
 
-    # ASTROPY_LT_40: astropy v4.0 introduced new default header keywords,
-    # once we support only astropy 4.0 and later we can update the reference
-    # data files and remove this section.
-    for key in (
-        "CRLN_OBS",
-        "CRLT_OBS",
-        "DSUN_OBS",
-        "HGLN_OBS",
-        "HGLT_OBS",
-        "MJDREFF",
-        "MJDREFI",
-        "MJDREF",
-        "MJD-OBS",
-        "RSUN_REF",
-    ):
-        header_out.pop(key, None)
     header_out["DATE-OBS"] = header_out["DATE-OBS"].replace("T", " ")
 
     return array_footprint_to_hdulist(output, footprint, header_out)
@@ -862,3 +814,28 @@ def test_broadcast_reprojection_conserve_flux(conserve_flux):
 @pytest.mark.parametrize("center_jacobian", (True, False))
 def test_broadcast_reprojection_center_jacobian(center_jacobian):
     _test_broadcast_reprojection_algo_specific_options(center_jacobian=center_jacobian)
+
+
+def test_adaptive_input_output_types(
+    valid_celestial_input_data, valid_celestial_output_projections
+):
+    # Check that all valid input/output types work properly
+
+    array_ref, wcs_in_ref, input_value, kwargs_in = valid_celestial_input_data
+
+    wcs_out_ref, shape_ref, output_value, kwargs_out = valid_celestial_output_projections
+
+    # Compute reference
+
+    output_ref, footprint_ref = reproject_adaptive(
+        (array_ref, wcs_in_ref), wcs_out_ref, shape_out=shape_ref
+    )
+
+    # Compute test
+
+    output_test, footprint_test = reproject_adaptive(
+        input_value, output_value, **kwargs_in, **kwargs_out
+    )
+
+    assert_allclose(output_ref, output_test)
+    assert_allclose(footprint_ref, footprint_test)
