@@ -3,7 +3,7 @@ import os
 
 from astropy.utils import deprecated_renamed_argument
 
-from ..utils import parse_input_data, parse_output_projection, reproject_blocked
+from ..utils import _reproject_blocked, parse_input_data, parse_output_projection
 from .core import _reproject_full
 
 __all__ = ["reproject_interp"]
@@ -39,13 +39,15 @@ def reproject_interp(
     input_data
         The input data to reproject. This can be:
 
-            * The name of a FITS file
+            * The name of a FITS file as a `str` or a `pathlib.Path` object
             * An `~astropy.io.fits.HDUList` object
             * An image HDU object such as a `~astropy.io.fits.PrimaryHDU`,
               `~astropy.io.fits.ImageHDU`, or `~astropy.io.fits.CompImageHDU`
               instance
             * A tuple where the first element is a `~numpy.ndarray` and the
-              second element is either a `~astropy.wcs.WCS` or a
+              second element is either a
+              `~astropy.wcs.wcsapi.BaseLowLevelWCS`,
+              `~astropy.wcs.wcsapi.BaseHighLevelWCS`, or a
               `~astropy.io.fits.Header` object
             * An `~astropy.nddata.NDData` object from which the ``.data`` and
               ``.wcs`` attributes will be used as the input data.
@@ -116,20 +118,7 @@ def reproject_interp(
 
     # if either of these are not default, it means a blocked method must be used
     if block_size is not None or parallel is not False:
-        # if parallel is set but block size isn't, we'll choose
-        # block size so each thread gets one block each
-        if parallel is not False and block_size is None:
-            block_size = list(shape_out)
-            # each thread gets an equal sized strip of output area to process
-            block_size[-2] = shape_out[-2] // os.cpu_count()
-
-        # given we have cases where modern system have many cpu cores some sanity clamping is
-        # to avoid 0 length block sizes when num_cpu_cores is greater than the side of the image
-        for dim_idx in range(min(len(shape_out), 2)):
-            if block_size[dim_idx] == 0:
-                block_size[dim_idx] = shape_out[dim_idx]
-
-        return reproject_blocked(
+        return _reproject_blocked(
             _reproject_full,
             array_in=array_in,
             wcs_in=wcs_in,
@@ -151,4 +140,5 @@ def reproject_interp(
             array_out=output_array,
             return_footprint=return_footprint,
             roundtrip_coords=roundtrip_coords,
+            output_footprint=output_footprint,
         )
