@@ -25,6 +25,7 @@ def as_high_level_wcs(wcs):
 @pytest.mark.array_compare(single_reference=True)
 @pytest.mark.parametrize("wcsapi", (False, True))
 @pytest.mark.parametrize("roundtrip_coords", (False, True))
+@pytest.mark.remote_data
 def test_reproject_celestial_2d_gal2equ(wcsapi, roundtrip_coords):
     """
     Test reprojection of a 2D celestial image, which includes a coordinate
@@ -65,6 +66,7 @@ for wcsapi in (False, True):
 @pytest.mark.array_compare(single_reference=True)
 @pytest.mark.parametrize(("wcsapi", "axis_order"), tuple(COMBINATIONS))
 @pytest.mark.parametrize("roundtrip_coords", (False, True))
+@pytest.mark.remote_data
 def test_reproject_celestial_3d_equ2gal(wcsapi, axis_order, roundtrip_coords):
     """
     Test reprojection of a 3D cube with celestial components, which includes a
@@ -118,6 +120,7 @@ def test_reproject_celestial_3d_equ2gal(wcsapi, axis_order, roundtrip_coords):
 @pytest.mark.array_compare(single_reference=True)
 @pytest.mark.parametrize("wcsapi", (False, True))
 @pytest.mark.parametrize("roundtrip_coords", (False, True))
+@pytest.mark.remote_data
 def test_small_cutout(wcsapi, roundtrip_coords):
     """
     Test reprojection of a cutout from a larger image (makes sure that the
@@ -151,6 +154,7 @@ def test_small_cutout(wcsapi, roundtrip_coords):
 
 
 @pytest.mark.parametrize("roundtrip_coords", (False, True))
+@pytest.mark.remote_data
 def test_mwpan_car_to_mol(roundtrip_coords):
     """
     Test reprojection of the Mellinger Milky Way Panorama from CAR to MOL,
@@ -183,6 +187,7 @@ def test_mwpan_car_to_mol(roundtrip_coords):
 
 
 @pytest.mark.parametrize("roundtrip_coords", (False, True))
+@pytest.mark.remote_data
 def test_small_cutout_outside(roundtrip_coords):
     """
     Test reprojection of a cutout from a larger image - in this case the
@@ -208,6 +213,7 @@ def test_small_cutout_outside(roundtrip_coords):
 
 
 @pytest.mark.parametrize("roundtrip_coords", (False, True))
+@pytest.mark.remote_data
 def test_celestial_mismatch_2d(roundtrip_coords):
     """
     Make sure an error is raised if the input image has celestial WCS
@@ -234,6 +240,7 @@ def test_celestial_mismatch_2d(roundtrip_coords):
 
 
 @pytest.mark.parametrize("roundtrip_coords", (False, True))
+@pytest.mark.remote_data
 def test_celestial_mismatch_3d(roundtrip_coords):
     """
     Make sure an error is raised if the input image has celestial WCS
@@ -270,6 +277,7 @@ def test_celestial_mismatch_3d(roundtrip_coords):
 
 
 @pytest.mark.parametrize("roundtrip_coords", (False, True))
+@pytest.mark.remote_data
 def test_spectral_mismatch_3d(roundtrip_coords):
     """
     Make sure an error is raised if there are mismatches between the presence
@@ -334,6 +342,7 @@ def test_naxis_mismatch(roundtrip_coords):
 
 
 @pytest.mark.parametrize("roundtrip_coords", (False, True))
+@pytest.mark.remote_data
 def test_slice_reprojection(roundtrip_coords):
     """
     Test case where only the slices change and the celestial projection doesn't
@@ -383,6 +392,7 @@ def test_slice_reprojection(roundtrip_coords):
 
 
 @pytest.mark.parametrize("roundtrip_coords", (False, True))
+@pytest.mark.remote_data
 def test_inequal_wcs_dims(roundtrip_coords):
     inp_cube = np.arange(3, dtype="float").repeat(4 * 5).reshape(3, 4, 5)
     header_in = fits.Header.fromtextfile(
@@ -406,8 +416,8 @@ def test_inequal_wcs_dims(roundtrip_coords):
 
 
 @pytest.mark.parametrize("roundtrip_coords", (False, True))
+@pytest.mark.remote_data
 def test_different_wcs_types(roundtrip_coords):
-
     inp_cube = np.arange(3, dtype="float").repeat(4 * 5).reshape(3, 4, 5)
     header_in = fits.Header.fromtextfile(
         get_pkg_data_filename("data/cube.hdr", package="reproject.tests")
@@ -435,8 +445,8 @@ def test_different_wcs_types(roundtrip_coords):
 
 
 @pytest.mark.parametrize("roundtrip_coords", (False, True))
+@pytest.mark.remote_data
 def test_reproject_3d_celestial_correctness_ra2gal(roundtrip_coords):
-
     inp_cube = np.arange(3, dtype="float").repeat(7 * 8).reshape(3, 7, 8)
 
     header_in = fits.Header.fromtextfile(
@@ -476,6 +486,7 @@ def test_reproject_3d_celestial_correctness_ra2gal(roundtrip_coords):
 
 
 @pytest.mark.parametrize("roundtrip_coords", (False, True))
+@pytest.mark.remote_data
 def test_reproject_with_output_array(roundtrip_coords):
     """
     Test both full_reproject and slicewise reprojection. We use a case where the
@@ -510,8 +521,8 @@ def test_reproject_with_output_array(roundtrip_coords):
 
 @pytest.mark.array_compare(single_reference=True)
 @pytest.mark.parametrize("file_format", ["fits", "asdf"])
+@pytest.mark.remote_data
 def test_reproject_roundtrip(file_format):
-
     # Test the reprojection with solar data, which ensures that the masking of
     # pixels based on round-tripping works correctly. Using asdf is not just
     # about testing a different format but making sure that GWCS works.
@@ -556,30 +567,14 @@ def test_reproject_roundtrip(file_format):
 
     header_out = target_wcs.to_header()
 
-    # ASTROPY_LT_40: astropy v4.0 introduced new default header keywords,
-    # once we support only astropy 4.0 and later we can update the reference
-    # data files and remove this section.
-    for key in (
-        "CRLN_OBS",
-        "CRLT_OBS",
-        "DSUN_OBS",
-        "HGLN_OBS",
-        "HGLT_OBS",
-        "MJDREFF",
-        "MJDREFI",
-        "MJDREF",
-        "MJD-OBS",
-        "RSUN_REF",
-    ):
-        header_out.pop(key, None)
     header_out["DATE-OBS"] = header_out["DATE-OBS"].replace("T", " ")
 
     return array_footprint_to_hdulist(output, footprint, header_out)
 
 
 @pytest.mark.parametrize("roundtrip_coords", (False, True))
+@pytest.mark.remote_data
 def test_identity_with_offset(roundtrip_coords):
-
     # Reproject an array and WCS to itself but with a margin, which should
     # end up empty. This is a regression test for a bug that caused some
     # values to extend beyond the original footprint.
@@ -670,10 +665,16 @@ def test_broadcast_reprojection(input_extra_dims, output_shape, input_as_wcs, ou
     np.testing.assert_allclose(array_broadcast, array_ref)
 
 
+# In the tests below we ignore FITSFixedWarning due to:
+# https://github.com/astropy/astropy/pull/12844
+
+
 @pytest.mark.parametrize("input_extra_dims", (1, 2))
 @pytest.mark.parametrize("output_shape", (None, "single", "full"))
 @pytest.mark.parametrize("parallel", [True, False])
-def test_blocked_broadcast_reprojection(input_extra_dims, output_shape, parallel):
+@pytest.mark.parametrize("header_or_wcs", (lambda x: x, WCS))
+@pytest.mark.filterwarnings("ignore::astropy.wcs.wcs.FITSFixedWarning")
+def test_blocked_broadcast_reprojection(input_extra_dims, output_shape, parallel, header_or_wcs):
     image_stack, array_ref, footprint_ref, header_in, header_out = _setup_for_broadcast_test()
     # Test both single and multiple dimensions being broadcast
     if input_extra_dims == 2:
@@ -689,27 +690,27 @@ def test_blocked_broadcast_reprojection(input_extra_dims, output_shape, parallel
         # Provide the broadcast dimensions as part of the output shape
         output_shape = image_stack.shape
 
-    # the warning import and ignore is needed to keep pytest happy when running with
-    # older versions of astropy which don't have this fix:
-    # https://github.com/astropy/astropy/pull/12844
-    # All the warning code should be removed when old version no longer being used
-    # Using context manager ensure only blocked function has them ignored
-    import warnings
+    # test different behavior when the output projection is a WCS
+    header_out = header_or_wcs(header_out)
 
-    with warnings.catch_warnings():
-        warnings.simplefilter("ignore", category=FITSFixedWarning)
-        array_broadcast, footprint_broadcast = reproject_interp(
-            (image_stack, header_in), header_out, output_shape, parallel=parallel, block_size=[5, 5]
-        )
+    array_broadcast, footprint_broadcast = reproject_interp(
+        (image_stack, header_in), header_out, output_shape, parallel=parallel, block_size=[5, 5]
+    )
 
     np.testing.assert_array_equal(footprint_broadcast, footprint_ref)
     np.testing.assert_allclose(array_broadcast, array_ref)
 
 
 @pytest.mark.parametrize("parallel", [True, 2, False])
-@pytest.mark.parametrize("block_size", [[40, 40], [500, 500], [500, 100], None])
-def test_blocked_against_single(parallel, block_size):
-
+@pytest.mark.parametrize("block_size", [[500, 500], [500, 100], None])
+@pytest.mark.parametrize("return_footprint", [False, True])
+@pytest.mark.parametrize("existing_outputs", [False, True])
+@pytest.mark.parametrize("header_or_wcs", (lambda x: x, WCS))
+@pytest.mark.remote_data
+@pytest.mark.filterwarnings("ignore::astropy.wcs.wcs.FITSFixedWarning")
+def test_blocked_against_single(
+    parallel, block_size, return_footprint, existing_outputs, header_or_wcs
+):
     # Ensure when we break a reprojection down into multiple discrete blocks
     # it has the same result as if all pixels where reprejcted at once
 
@@ -718,81 +719,76 @@ def test_blocked_against_single(parallel, block_size):
     array_test = None
     footprint_test = None
 
-    # the warning import and ignore is needed to keep pytest happy when running with
-    # older versions of astropy which don't have this fix:
-    # https://github.com/astropy/astropy/pull/12844
-    # All the warning code should be removed when old version no longer being used
-    # Using context manager ensure only blocked function has them ignored
-    import warnings
+    shape_out = (720, 721)
 
-    with warnings.catch_warnings():
-        warnings.simplefilter("ignore", category=FITSFixedWarning)
+    if existing_outputs:
+        output_array_test = np.zeros(shape_out)
+        output_footprint_test = np.zeros(shape_out)
+        output_array_reference = np.zeros(shape_out)
+        output_footprint_reference = np.zeros(shape_out)
+    else:
+        output_array_test = None
+        output_footprint_test = None
+        output_array_reference = None
+        output_footprint_reference = None
 
-        # this one is needed to avoid the following warning from when the np.as_strided() is
-        # called in wcs_utils.unbroadcast(), only shows up with py3.8, numpy1.17, astropy 4.0.*:
-        #     DeprecationWarning: Numpy has detected that you (may be) writing to an array with
-        #     overlapping memory from np.broadcast_arrays. If this is intentional
-        #     set the WRITEABLE flag True or make a copy immediately before writing.
-        # We do call as_strided with writeable=True as it recommends and only shows up with the 10px
-        # testcase so assuming a numpy bug in the detection code which was fixed in later version.
-        # The pixel values all still match in the end, only shows up due to pytest clearing
-        # the standard python warning filters by default and failing as the warnings are now
-        # treated as the exceptions they're implemented on
-        if block_size == [10, 10]:
-            warnings.simplefilter("ignore", category=DeprecationWarning)
-
-        array_test, footprint_test = reproject_interp(
-            hdu2, hdu1.header, parallel=parallel, block_size=block_size
-        )
-
-    array_reference, footprint_reference = reproject_interp(
-        hdu2, hdu1.header, parallel=False, block_size=None
+    result_test = reproject_interp(
+        hdu2,
+        header_or_wcs(hdu1.header),
+        parallel=parallel,
+        block_size=block_size,
+        return_footprint=return_footprint,
+        output_array=output_array_test,
+        output_footprint=output_footprint_test,
     )
 
+    result_reference = reproject_interp(
+        hdu2,
+        header_or_wcs(hdu1.header),
+        parallel=False,
+        block_size=None,
+        return_footprint=return_footprint,
+        output_array=output_array_reference,
+        output_footprint=output_footprint_reference,
+    )
+
+    if return_footprint:
+        array_test, footprint_test = result_test
+        array_reference, footprint_reference = result_reference
+    else:
+        array_test = result_test
+        array_reference = result_reference
+
+    if existing_outputs:
+        assert array_test is output_array_test
+        assert array_reference is output_array_reference
+        if return_footprint:
+            assert footprint_test is output_footprint_test
+            assert footprint_reference is output_footprint_reference
+
     np.testing.assert_allclose(array_test, array_reference, equal_nan=True)
-    np.testing.assert_allclose(footprint_test, footprint_reference, equal_nan=True)
+    if return_footprint:
+        np.testing.assert_allclose(footprint_test, footprint_reference, equal_nan=True)
 
 
-def test_blocked_corner_cases():
+def test_interp_input_output_types(valid_celestial_input_data, valid_celestial_output_projections):
+    # Check that all valid input/output types work properly
 
-    """
-    When doing blocked there are a few checks designed to sanity clamp/preserve
-    values. Even though the blocking process only tiles in a 2d manner 3d information
-    about the image needs to be preserved and transformed correctly. Additonally
-    when automatically determining block size based on CPU cores zeros can appear on
-    machines where num_cores > x or y dim of output image. So make sure it correctly
-    functions when 0 block size goes in
-    """
+    array_ref, wcs_in_ref, input_value, kwargs_in = valid_celestial_input_data
 
-    # Read in the input cube
-    hdu_in = fits.open(get_pkg_data_filename("data/equatorial_3d.fits", package="reproject.tests"))[
-        0
-    ]
+    wcs_out_ref, shape_ref, output_value, kwargs_out = valid_celestial_output_projections
 
-    # Define the output header - this should be the same for all versions of
-    # this test to make sure we can use a single reference file.
-    header_out = hdu_in.header.copy()
-    header_out["NAXIS1"] = 10
-    header_out["NAXIS2"] = 9
-    header_out["CTYPE1"] = "GLON-SIN"
-    header_out["CTYPE2"] = "GLAT-SIN"
-    header_out["CRVAL1"] = 163.16724
-    header_out["CRVAL2"] = -15.777405
-    header_out["CRPIX1"] = 6
-    header_out["CRPIX2"] = 5
+    # Compute reference
 
-    array_reference = reproject_interp(hdu_in, header_out, return_footprint=False)
+    output_ref, footprint_ref = reproject_interp(
+        (array_ref, wcs_in_ref), wcs_out_ref, shape_out=shape_ref
+    )
 
-    array_test = None
+    # Compute test
 
-    # same reason as test above for FITSFixedWarning
-    import warnings
+    output_test, footprint_test = reproject_interp(
+        input_value, output_value, **kwargs_in, **kwargs_out
+    )
 
-    with warnings.catch_warnings():
-        warnings.simplefilter("ignore", category=FITSFixedWarning)
-
-        array_test = reproject_interp(
-            hdu_in, header_out, parallel=True, block_size=[0, 4], return_footprint=False
-        )
-
-    np.testing.assert_allclose(array_test, array_reference, equal_nan=True, verbose=True)
+    assert_allclose(output_ref, output_test)
+    assert_allclose(footprint_ref, footprint_test)
