@@ -1,6 +1,8 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
 import numpy as np
+from astropy.wcs import WCS
+from astropy.wcs.wcsapi import SlicedLowLevelWCS
 
 from ..utils import parse_input_data, parse_input_weights, parse_output_projection
 from .background import determine_offset_matrix, solve_corrections_sgd
@@ -109,7 +111,6 @@ def reproject_and_coadd(
     arrays = []
 
     for idata in range(len(input_data)):
-
         # We need to pre-parse the data here since we need to figure out how to
         # optimize/minimize the size of each output tile (see below).
         array_in, wcs_in = parse_input_data(input_data[idata], hdu_in=hdu_in)
@@ -155,7 +156,13 @@ def reproject_and_coadd(
         if imax < imin or jmax < jmin:
             continue
 
-        wcs_out_indiv = wcs_out[jmin:jmax, imin:imax]
+        if isinstance(wcs_out, WCS):
+            wcs_out_indiv = wcs_out[jmin:jmax, imin:imax]
+        else:
+            wcs_out_indiv = SlicedLowLevelWCS(
+                wcs_out.low_level_wcs, (slice(jmin, jmax), slice(imin, imax))
+            )
+
         shape_out_indiv = (jmax - jmin, imax - imin)
 
         # TODO: optimize handling of weights by making reprojection functions
@@ -214,9 +221,7 @@ def reproject_and_coadd(
     final_footprint = np.zeros(shape_out)
 
     if combine_function in ("mean", "sum"):
-
         for array in arrays:
-
             # By default, values outside of the footprint are set to NaN
             # but we set these to 0 here to avoid getting NaNs in the
             # means/sums.
@@ -230,7 +235,6 @@ def reproject_and_coadd(
                 final_array /= final_footprint
 
     elif combine_function == "median":
-
         # Here we need to operate in chunks since we could otherwise run
         # into memory issues
 
