@@ -14,7 +14,6 @@ from ..high_level import reproject_exact
 
 class TestReprojectExact:
     def setup_class(self):
-
         header_gal = get_pkg_data_filename("../../tests/data/gc_ga.hdr")
         header_equ = get_pkg_data_filename("../../tests/data/gc_eq.hdr")
         self.header_in = fits.Header.fromtextfile(header_gal)
@@ -36,7 +35,6 @@ class TestReprojectExact:
         reproject_exact((self.array_in, self.header_in), self.header_out)
 
     def test_parallel_option(self):
-
         reproject_exact((self.array_in, self.header_in), self.header_out, parallel=1)
 
         with pytest.raises(ValueError) as exc:
@@ -45,7 +43,6 @@ class TestReprojectExact:
 
 
 def test_identity():
-
     # Reproject an array and WCS to itself
 
     wcs = WCS(naxis=2)
@@ -64,9 +61,7 @@ def test_identity():
 
 
 def test_reproject_precision_warning():
-
     for res in [0.1 / 3600, 0.01 / 3600]:
-
         wcs1 = WCS()
         wcs1.wcs.ctype = "RA---TAN", "DEC--TAN"
         wcs1.wcs.crval = 13, 80
@@ -182,3 +177,26 @@ def test_broadcast_parallel_reprojection(input_extra_dims, output_shape, paralle
 
     np.testing.assert_allclose(footprint_broadcast, footprint_ref)
     np.testing.assert_allclose(array_broadcast, array_ref)
+
+
+def test_exact_input_output_types(valid_celestial_input_data, valid_celestial_output_projections):
+    # Check that all valid input/output types work properly
+
+    array_ref, wcs_in_ref, input_value, kwargs_in = valid_celestial_input_data
+
+    wcs_out_ref, shape_ref, output_value, kwargs_out = valid_celestial_output_projections
+
+    # Compute reference
+
+    output_ref, footprint_ref = reproject_exact(
+        (array_ref, wcs_in_ref), wcs_out_ref, shape_out=shape_ref
+    )
+
+    # Compute test
+
+    output_test, footprint_test = reproject_exact(
+        input_value, output_value, **kwargs_in, **kwargs_out
+    )
+
+    assert_allclose(output_ref, output_test)
+    assert_allclose(footprint_ref, footprint_test)
