@@ -1,12 +1,17 @@
+import tempfile
+from pathlib import Path
 from concurrent import futures
 
 import astropy.nddata
+import dask
+import dask.array as da
 import numpy as np
 from astropy.io import fits
 from astropy.io.fits import CompImageHDU, HDUList, Header, ImageHDU, PrimaryHDU
 from astropy.wcs import WCS
-from astropy.wcs.wcsapi import BaseHighLevelWCS, SlicedLowLevelWCS
+from astropy.wcs.wcsapi import BaseHighLevelWCS, BaseLowLevelWCS, SlicedLowLevelWCS
 from astropy.wcs.wcsapi.high_level_wcs_wrapper import HighLevelWCSWrapper
+from dask.utils import SerializableLock
 
 __all__ = [
     "parse_input_data",
@@ -16,12 +21,53 @@ __all__ = [
 ]
 
 
+def _dask_to_numpy_memmap(dask_array):
+    """
+    Given a dask array, write it out to disk and load it again as a Numpy
+    memmap array.
+    """
+
+    with tempfile.TemporaryDirectory() as zarr_tmp:
+        # First compute and store the dask array to zarr using whatever
+        # the default scheduler is at this point
+        dask_array.to_zarr(zarr_tmp)
+
+        # Load the array back to dask
+        zarr_array = da.from_zarr(zarr_tmp)
+
+        # Then store this in the Numpy memmap array (schedulers other than
+        # synchronous don't work well when writing to a Numpy array)
+
+        memmap_path = tempfile.mktemp()
+
+        memmapped_array = np.memmap(
+            memmap_path,
+            dtype=zarr_array.dtype,
+            shape=zarr_array.shape,
+            mode="w+",
+        )
+
+        da.store(
+            zarr_array,
+            memmapped_array,
+            compute=True,
+            scheduler="synchronous",
+        )
+
+    # Note that at this point the zarr directory should be removed, but
+    # the memmapped array will persist on disk. In future we may want to
+    # clean this up automatically once reproject has completed to avoid
+    # a build-up of temporary files
+
+    return memmapped_array
+
+
 def parse_input_data(input_data, hdu_in=None):
     """
     Parse input data to return a Numpy array and WCS object.
     """
 
-    if isinstance(input_data, str):
+    if isinstance(input_data, (str, Path)):
         with fits.open(input_data) as hdul:
             return parse_input_data(hdul, hdu_in=hdu_in)
     elif isinstance(input_data, HDUList):
@@ -36,11 +82,25 @@ def parse_input_data(input_data, hdu_in=None):
         return parse_input_data(input_data[hdu_in])
     elif isinstance(input_data, (PrimaryHDU, ImageHDU, CompImageHDU)):
         return input_data.data, WCS(input_data.header)
-    elif isinstance(input_data, tuple) and isinstance(input_data[0], np.ndarray):
-        if isinstance(input_data[1], Header):
-            return input_data[0], WCS(input_data[1])
+    elif isinstance(input_data, tuple) and isinstance(input_data[0], (np.ndarray, da.core.Array)):
+        if isinstance(input_data[0], da.core.Array):
+            data = _dask_to_numpy_memmap(input_data[0])
         else:
-            return input_data
+            data = input_data[0]
+        if isinstance(input_data[1], Header):
+            return data, WCS(input_data[1])
+        else:
+            if isinstance(input_data[1], BaseHighLevelWCS):
+                return data, input_data[1]
+            else:
+                return data, HighLevelWCSWrapper(input_data[1])
+    elif (
+        isinstance(input_data, BaseHighLevelWCS)
+        and input_data.low_level_wcs.array_shape is not None
+    ):
+        return input_data.array_shape, input_data
+    elif isinstance(input_data, BaseLowLevelWCS) and input_data.array_shape is not None:
+        return input_data.array_shape, HighLevelWCSWrapper(input_data)
     elif isinstance(input_data, astropy.nddata.NDDataBase):
         return input_data.data, input_data.wcs
     else:
@@ -55,7 +115,7 @@ def parse_input_shape(input_shape, hdu_in=None):
     Parse input shape information to return an array shape tuple and WCS object.
     """
 
-    if isinstance(input_shape, str):
+    if isinstance(input_shape, (str, Path)):
         return parse_input_shape(fits.open(input_shape), hdu_in=hdu_in)
     elif isinstance(input_shape, HDUList):
         if hdu_in is None:
@@ -69,16 +129,29 @@ def parse_input_shape(input_shape, hdu_in=None):
         return parse_input_shape(input_shape[hdu_in])
     elif isinstance(input_shape, (PrimaryHDU, ImageHDU, CompImageHDU)):
         return input_shape.shape, WCS(input_shape.header)
-    elif isinstance(input_shape, tuple) and isinstance(input_shape[0], np.ndarray):
+    elif isinstance(input_shape, tuple) and isinstance(input_shape[0], (np.ndarray, da.core.Array)):
         if isinstance(input_shape[1], Header):
             return input_shape[0].shape, WCS(input_shape[1])
         else:
-            return input_shape[0].shape, input_shape[1]
+            if isinstance(input_shape[1], BaseHighLevelWCS):
+                return input_shape[0].shape, input_shape[1]
+            else:
+                return input_shape[0].shape, HighLevelWCSWrapper(input_shape[1])
     elif isinstance(input_shape, tuple) and isinstance(input_shape[0], tuple):
         if isinstance(input_shape[1], Header):
             return input_shape[0], WCS(input_shape[1])
         else:
-            return input_shape
+            if isinstance(input_shape[1], BaseHighLevelWCS):
+                return input_shape
+            else:
+                return input_shape[0], HighLevelWCSWrapper(input_shape[1])
+    elif (
+        isinstance(input_shape, BaseHighLevelWCS)
+        and input_shape.low_level_wcs.array_shape is not None
+    ):
+        return input_shape.low_level_wcs.array_shape, input_shape
+    elif isinstance(input_shape, BaseLowLevelWCS) and input_shape.array_shape is not None:
+        return input_shape.array_shape, HighLevelWCSWrapper(input_shape)
     elif isinstance(input_shape, astropy.nddata.NDDataBase):
         return input_shape.data.shape, input_shape.wcs
     else:
@@ -114,7 +187,6 @@ def parse_input_weights(input_weights, hdu_weights=None):
 
 
 def parse_output_projection(output_projection, shape_in=None, shape_out=None, output_array=None):
-
     if shape_out is None:
         if output_array is not None:
             shape_out = output_array.shape
@@ -134,12 +206,18 @@ def parse_output_projection(output_projection, shape_in=None, shape_out=None, ou
                     "Need to specify shape since output header "
                     "does not contain complete shape information"
                 )
-    elif isinstance(output_projection, BaseHighLevelWCS):
-        wcs_out = output_projection
+    elif isinstance(output_projection, (BaseLowLevelWCS, BaseHighLevelWCS)):
+        if isinstance(output_projection, BaseLowLevelWCS):
+            wcs_out = HighLevelWCSWrapper(output_projection)
+        else:
+            wcs_out = output_projection
         if shape_out is None:
-            raise ValueError(
-                "Need to specify shape_out when specifying output_projection as WCS object"
-            )
+            if wcs_out.low_level_wcs.array_shape is not None:
+                shape_out = wcs_out.low_level_wcs.array_shape
+            else:
+                raise ValueError(
+                    "Need to specify shape_out when specifying output_projection as WCS object"
+                )
     elif isinstance(output_projection, str):
         hdu_list = fits.open(output_projection)
         shape_out = hdu_list[0].data.shape
@@ -160,53 +238,10 @@ def parse_output_projection(output_projection, shape_in=None, shape_out=None, ou
         # Add the broadcast dimensions to the output shape, which does not
         # currently have any broadcast dims
         shape_out = (*shape_in[: -len(shape_out)], *shape_out)
-    return wcs_out, shape_out
+    return wcs_out, tuple(shape_out)
 
 
-def _block(
-    reproject_func, array_in, wcs_in, wcs_out_sub, shape_out, i_range, j_range, return_footprint
-):
-    """
-    Implementation function that handles reprojecting subsets blocks of pixels
-    from an input image and holds metadata about where to reinsert when done.
-
-    Parameters
-    ----------
-    reproject_func
-        One the existing reproject functions implementing a reprojection algorithm
-        that that will be used be used to perform reprojection
-    array_in
-        Data following the same format as expected by underlying reproject_func,
-        expected to `~numpy.ndarray` when used from reproject_blocked()
-    wcs_in: `~astropy.wcs.WCS`
-        WCS object corresponding to array_in
-    wcs_out_sub:
-        Output WCS image will be projected to. Normally will correspond to subset of
-        total output image when used by repoject_blocked()
-    shape_out:
-        Passed to reproject_func() alongside WCS out to determine image size
-    i_range:
-        Passed through unmodified, used to determine where to reinsert block
-    j_range:
-        Passed through unmodified, used to determine where to reinsert block
-    """
-
-    result = reproject_func(
-        array_in, wcs_in, wcs_out_sub, shape_out=shape_out, return_footprint=return_footprint
-    )
-
-    res_arr = None
-    res_fp = None
-
-    if return_footprint:
-        res_arr, res_fp = result
-    else:
-        res_arr = result
-
-    return {"i": i_range, "j": j_range, "res_arr": res_arr, "res_fp": res_fp}
-
-
-def reproject_blocked(
+def _reproject_blocked(
     reproject_func,
     array_in,
     wcs_in,
@@ -229,14 +264,14 @@ def reproject_blocked(
         that that will be used be used to perform reprojection
     array_in
         Data following the same format as expected by underlying reproject_func,
-        expected to `~numpy.ndarray` when used from reproject_blocked()
+        expected to `~numpy.ndarray` when used from _reproject_blocked()
     wcs_in: `~astropy.wcs.WCS`
         WCS object corresponding to array_in
     shape_out: tuple
         Passed to reproject_func() alongside WCS out to determine image size
     wcs_out: `~astropy.wcs.WCS`
         Output WCS image will be projected to. Normally will correspond to subset of
-        total output image when used by repoject_blocked()
+        total output image when used by _reproject_blocked()
     block_size: tuple
         The size of blocks in terms of output array pixels that each block will handle
         reprojecting. Extending out from (0,0) coords positively, block sizes
@@ -264,96 +299,90 @@ def reproject_blocked(
     if output_footprint is None and return_footprint:
         output_footprint = np.zeros(shape_out, dtype=float)
 
-    # setup variables needed for multiprocessing if required
-    proc_pool = None
-    blocks_futures = []
+    if block_size is not None and len(block_size) < len(shape_out):
+        block_size = [-1] * (len(shape_out) - len(block_size)) + list(block_size)
 
-    if parallel or type(parallel) is int:
-        if type(parallel) is int:
-            if parallel <= 0:
-                raise ValueError("The number of processors to use must be strictly positive")
-            else:
-                proc_pool = futures.ProcessPoolExecutor(max_workers=parallel)
+    shape_in = array_in.shape
+
+    # When in parallel mode, we want to make sure we avoid having to copy the
+    # input array to all processes for each chunk, so instead we write out
+    # the input array to a Numpy memory map and load it in inside each process
+    # as a memory-mapped array. We need to be careful how this gets passed to
+    # reproject_single_block so we pass a variable that can be either a string
+    # or the array itself (for synchronous mode).
+    if parallel:
+        array_in_or_path = tempfile.mktemp()
+        array_in_memmapped = np.memmap(
+            array_in_or_path, dtype=float, shape=array_in.shape, mode="w+"
+        )
+        array_in_memmapped[:] = array_in[:]
+    else:
+        array_in_or_path = array_in
+
+    def reproject_single_block(a, block_info=None):
+        if a.ndim == 0 or block_info is None or block_info == []:
+            return np.array([a, a])
+        slices = [slice(*x) for x in block_info[None]["array-location"][-wcs_out.pixel_n_dim :]]
+
+        if isinstance(wcs_out, BaseHighLevelWCS):
+            low_level_wcs = SlicedLowLevelWCS(wcs_out.low_level_wcs, slices=slices)
         else:
-            proc_pool = futures.ProcessPoolExecutor()
+            low_level_wcs = SlicedLowLevelWCS(wcs_out, slices=slices)
+        wcs_out_sub = HighLevelWCSWrapper(low_level_wcs)
+        if isinstance(array_in_or_path, str):
+            array_in = np.memmap(array_in_or_path, dtype=float, shape=shape_in)
+        else:
+            array_in = array_in_or_path
+        array, footprint = reproject_func(
+            array_in, wcs_in, wcs_out_sub, block_info[None]["chunk-shape"][1:]
+        )
+        return np.array([array, footprint])
 
-    # This will iterate over the output space, generating slices of that
-    # WCS and either processing and reinserting them immediately,
-    # or when doing parallel impl submit them to workers then wait and reinsert as
-    # the workers complete each block
-    for imin in range(0, output_array.shape[-2], block_size[0]):
-        imax = min(imin + block_size[0], output_array.shape[-2])
-        for jmin in range(0, output_array.shape[-1], block_size[1]):
-            jmax = min(jmin + block_size[1], output_array.shape[-1])
-            shape_out_sub = (imax - imin, jmax - jmin)
-            # if the output has more than two dims, apply our blocking to only the last two
-            shape_out_sub = output_array.shape[:-2] + shape_out_sub
+    # NOTE: the following array is just used to set up the iteration in map_blocks
+    # but isn't actually used otherwise - this is deliberate.
+    output_array_dask = da.empty(shape_out, chunks=block_size or "auto")
 
-            slices = [slice(imin, imax), slice(jmin, jmax)]
-            if wcs_out.low_level_wcs.pixel_n_dim > 2:
-                slices = [Ellipsis] + slices
-            wcs_out_sub = HighLevelWCSWrapper(SlicedLowLevelWCS(wcs_out, slices=slices))
+    result = da.map_blocks(
+        reproject_single_block,
+        output_array_dask,
+        dtype=float,
+        new_axis=0,
+        chunks=(2,) + output_array_dask.chunksize,
+    )
 
-            if proc_pool is None:
-                # if sequential input data and reinsert block into main array immediately
-                completed_block = _block(
-                    reproject_func=reproject_func,
-                    array_in=array_in,
-                    wcs_in=wcs_in,
-                    wcs_out_sub=wcs_out_sub,
-                    shape_out=shape_out_sub,
-                    return_footprint=return_footprint,
-                    j_range=(jmin, jmax),
-                    i_range=(imin, imax),
-                )
+    # Truncate extra elements
+    result = result[tuple([slice(None)] + [slice(s) for s in shape_out])]
 
-                output_array[..., imin:imax, jmin:jmax] = completed_block["res_arr"][:]
-                if return_footprint:
-                    output_footprint[..., imin:imax, jmin:jmax] = completed_block["res_fp"][:]
-
-            else:
-                # if parallel just submit all work items and move on to waiting for them to be done
-                future = proc_pool.submit(
-                    _block,
-                    reproject_func=reproject_func,
-                    array_in=array_in,
-                    wcs_in=wcs_in,
-                    wcs_out_sub=wcs_out_sub,
-                    shape_out=shape_out_sub,
-                    return_footprint=return_footprint,
-                    j_range=(jmin, jmax),
-                    i_range=(imin, imax),
-                )
-                blocks_futures.append(future)
-
-    # If a parallel implementation is being used that means the
-    # blocks have not been reassembled yet and must be done now as their
-    # block call completes in the worker processes
-    if proc_pool is not None:
-        completed_future_count = 0
-        for completed_future in futures.as_completed(blocks_futures):
-            completed_block = completed_future.result()
-            i_range = completed_block["i"]
-            j_range = completed_block["j"]
-            output_array[..., i_range[0] : i_range[1], j_range[0] : j_range[1]] = completed_block[
-                "res_arr"
-            ][:]
-
-            if return_footprint:
-                footprint_block = completed_block["res_fp"][:]
-                output_footprint[
-                    ..., i_range[0] : i_range[1], j_range[0] : j_range[1]
-                ] = footprint_block
-
-            completed_future_count += 1
-            idx = blocks_futures.index(completed_future)
-            # ensure memory used by returned data is freed
-            completed_future._result = None
-            del blocks_futures[idx], completed_future
-        proc_pool.shutdown()
-        del blocks_futures
+    if parallel:
+        # As discussed in https://github.com/dask/dask/issues/9556, da.store
+        # will not work well in multiprocessing mode when the destination is a
+        # Numpy array. Instead, in this case we save the dask array to a zarr
+        # array on disk which can be done in parallel, and re-load it as a dask
+        # array. We can then use da.store in the next step using the
+        # 'synchronous' scheduler since that is I/O limited so does not need
+        # to be done in parallel.
+        filename = tempfile.mktemp()
+        if isinstance(parallel, int):
+            workers = {"num_workers": parallel}
+        else:
+            workers = {}
+        with dask.config.set(scheduler="processes", **workers):
+            result.to_zarr(filename)
+        result = da.from_zarr(filename)
 
     if return_footprint:
+        da.store(
+            [result[0], result[1]],
+            [output_array, output_footprint],
+            compute=True,
+            scheduler="synchronous",
+        )
         return output_array, output_footprint
     else:
+        da.store(
+            result[0],
+            output_array,
+            compute=True,
+            scheduler="synchronous",
+        )
         return output_array
